@@ -6,12 +6,13 @@
 
 package MCE::Shared::Array;
 
+use 5.010001;
 use strict;
 use warnings;
 
 no warnings qw( threads recursion uninitialized numeric );
 
-our $VERSION = '1.001';
+our $VERSION = '1.002';
 
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
@@ -366,7 +367,6 @@ sub len {
    *{ __PACKAGE__.'::shift'   } = \&SHIFT;
    *{ __PACKAGE__.'::unshift' } = \&UNSHIFT;
    *{ __PACKAGE__.'::splice'  } = \&SPLICE;
-
    *{ __PACKAGE__.'::del'     } = \&delete;
    *{ __PACKAGE__.'::merge'   } = \&mset;
    *{ __PACKAGE__.'::vals'    } = \&values;
@@ -388,7 +388,7 @@ MCE::Shared::Array - Array helper class
 
 =head1 VERSION
 
-This document describes MCE::Shared::Array version 1.001
+This document describes MCE::Shared::Array version 1.002
 
 =head1 SYNOPSIS
 
@@ -437,8 +437,8 @@ This document describes MCE::Shared::Array version 1.001
 
    # search capability key/val { =~ !~ eq ne lt le gt ge == != < <= > >= }
    # key/val means to match against actual key/val respectively
-   # do not add quotes inside the string unless intended literally
-   # do not mix :AND(s) and :OR(s) together
+
+   @keys  = $ar->keys( "key == 3 :or (val > 5 :and val < 9)" );
 
    @keys  = $ar->keys( "key =~ /$pattern/i" );
    @keys  = $ar->keys( "key !~ /$pattern/i" );
@@ -446,16 +446,17 @@ This document describes MCE::Shared::Array version 1.001
    @keys  = $ar->keys( "val !~ /$pattern/i" );
 
    %pairs = $ar->pairs( "key == $number" );
-   %pairs = $ar->pairs( "key != $number :AND val > 100" );
-   %pairs = $ar->pairs( "key <  $number :OR key > $number" );
+   %pairs = $ar->pairs( "key != $number :and val > 100" );
+   %pairs = $ar->pairs( "key <  $number :or key > $number" );
    %pairs = $ar->pairs( "val <= $number" );
    %pairs = $ar->pairs( "val >  $number" );
    %pairs = $ar->pairs( "val >= $number" );
 
    @vals  = $ar->values( "key eq $string" );
    @vals  = $ar->values( "key ne $string with space" );
-   @vals  = $ar->values( "key lt $string :OR val =~ /$pat1|$pat2/" );
-   @vals  = $ar->values( "val le $string :AND val eq foo bar" );
+   @vals  = $ar->values( "key lt $string :or val =~ /$pat1|$pat2/" );
+   @vals  = $ar->values( "val le $string :and val eq 'foo bar'" );
+   @vals  = $ar->values( "val le $string :and val eq foo bar" );
    @vals  = $ar->values( "val gt $string" );
    @vals  = $ar->values( "val ge $string" );
 
@@ -477,16 +478,17 @@ An array helper class for use with L<MCE::Shared>.
 =head1 SYNTAX for QUERY STRING
 
 Several methods in C<MCE::Shared::Array> take a query string for an argument.
-The format of the string is quoteless. Therefore, any quotes inside the string
-is treated literally.
 
    o Basic demonstration: @keys = $ar->keys( "val =~ /pattern/" );
    o Supported operators: =~ !~ eq ne lt le gt ge == != < <= > >=
-   o Multiple expressions are delimited by :AND or :OR.
+   o Multiple expressions delimited by :AND or :OR
+   o Quoting optional inside the string
 
-     "key =~ /pattern/i :AND val =~ /pattern/i"
-     "key =~ /pattern/i :AND val eq foo bar"     # val eq "foo bar"
-     "val eq foo baz :OR key !~ /pattern/i"
+     "key == 3 :or (val > 5 :and val < 9)"
+     "key =~ /pattern/i :and val =~ /pattern/i"
+     "key =~ /pattern/i :and val eq 'foo bar'"   # val eq "foo bar"
+     "key =~ /pattern/i :and val eq foo bar"     # val eq "foo bar"
+     "val eq foo baz :or key !~ /pattern/i"
 
      * key matches on indices in the array
      * val matches on values
@@ -495,8 +497,6 @@ is treated literally.
 
 =item * The modifiers C<:AND> and C<:OR> may be mixed case. e.g. C<:And>
 
-=item * Mixing C<:AND> and C<:OR> in the query is not supported.
-
 =back
 
 =head1 API DOCUMENTATION
